#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


from fanstatic import Library
library = Library('pyams_security', 'resources')

from pyramid.i18n import TranslationStringFactory
_ = TranslationStringFactory('pyams_security')

from pyams_utils.interfaces import PUBLIC_PERMISSION, VIEW_PERMISSION, MANAGE_PERMISSION, VIEW_SYSTEM_PERMISSION, \
    MANAGE_SYSTEM_PERMISSION, MANAGE_SECURITY_PERMISSION, MANAGE_ROLES_PERMISSION


def includeme(config):
    """Pyramid include

    Split in another package to remove cyclic dependencies with TranslationStringFactory
    """
    from .include import include_package
    include_package(config)

    # register custom permissions
    config.register_permission({'id': PUBLIC_PERMISSION,
                                'title': _("View public contents")})
    config.register_permission({'id': VIEW_PERMISSION,
                                'title': _("View protected contents")})
    config.register_permission({'id': MANAGE_PERMISSION,
                                'title': _("Manage contents properties")})
    config.register_permission({'id': VIEW_SYSTEM_PERMISSION,
                                'title': _("View management screens")})
    config.register_permission({'id': MANAGE_SYSTEM_PERMISSION,
                                'title': _("Manage system properties")})

    config.register_permission({'id': MANAGE_SECURITY_PERMISSION,
                                'title': _("Manage security")})
    config.register_permission({'id': MANAGE_ROLES_PERMISSION,
                                'title': _("Manage roles")})

    # register custom roles
    config.register_role({'id': 'system.Manager',
                          'title': "System manager (role)",
                          'permissions': {PUBLIC_PERMISSION, VIEW_PERMISSION, MANAGE_PERMISSION,
                                          MANAGE_SYSTEM_PERMISSION, VIEW_SYSTEM_PERMISSION,
                                          MANAGE_SECURITY_PERMISSION, MANAGE_ROLES_PERMISSION},
                          'managers': {'system:admin', 'role:system.Manager'}})
