#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_security.interfaces import IPrincipalInfo
from pyams_security.interfaces.profile import PUBLIC_PROFILE_KEY, IPublicProfile
from pyams_utils.interfaces.site import ISiteRoot
from pyams_utils.interfaces.tales import ITALESExtension
from zope.annotation.interfaces import IAnnotations, IAttributeAnnotatable
from zope.traversing.interfaces import ITraversable

# import packages
from persistent import Persistent
from pyams_file.property import FileProperty
from pyams_utils.adapter import adapter_config, ContextRequestAdapter
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyramid.threadlocal import get_current_registry, get_current_request
from zope.container.contained import Contained
from zope.lifecycleevent import ObjectCreatedEvent
from zope.interface import implementer, Interface
from zope.location import locate


@implementer(IPublicProfile, IAttributeAnnotatable)
class PublicProfile(Persistent, Contained):
    """Public profile persistent class"""

    avatar = FileProperty(IPublicProfile['avatar'])


@adapter_config(context=Interface, provides=IPublicProfile)
def PublicProfileFactory(context):
    request = check_request()
    return IPublicProfile(request.principal)


@adapter_config(context=IPrincipalInfo, provides=IPublicProfile)
def PrincipalPublicProfileFactory(principal):
    """Principal public profile factory adapter"""
    annotations = IAnnotations(principal)
    profile = annotations.get(PUBLIC_PROFILE_KEY)
    if profile is None:
        profile = annotations[PUBLIC_PROFILE_KEY] = PublicProfile()
        get_current_registry().notify(ObjectCreatedEvent(profile))
        request = get_current_request()
        if request is not None:
            root = get_parent(request.context, ISiteRoot)
            locate(profile, root, '++profile++')
    return profile


@adapter_config(name='profile', context=(Interface, Interface), provides=ITraversable)
class ProfileTraverser(ContextRequestAdapter):
    """++profile++ namespace traverser"""

    def traverse(self, name, furtherpath=None):
        return IPublicProfile(self.request)


@adapter_config(name='public_profile', context=(Interface, Interface), provides=ITALESExtension)
class PublicProfileExtension(ContextRequestAdapter):
    """public_profile TALES extension"""

    def render(self, request=None):
        if request is None:
            request = self.request
        return IPublicProfile(request)
