#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_form.interfaces.form import IInnerTabForm
from pyams_security.interfaces.profile import IPublicProfile
from pyams_skin.interfaces.viewlet import IShortcutsViewletManager
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_form.form import InnerEditForm, AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_skin.viewlet.shortcuts import Shortcut
from pyams_utils.adapter import adapter_config
from pyams_zmi.form import AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface

from pyams_security import _


@viewlet_config(name='profile', layer=IPyAMSLayer, manager=IShortcutsViewletManager, weight=10)
class UserProfileShortcut(Shortcut):
    """User profile shortcut"""

    label = _("User profile")
    bg_color_class = 'bg-color-greenLight'
    icon_class = 'fa-user'
    url = 'user-profile.html'
    modal_target = True
    checked = 'selected'


@pagelet_config(name='user-profile.html', layer=IPyAMSLayer)
class UserProfileEditForm(AdminDialogEditForm):
    """User profile edit form"""

    legend = _("Edit user profile")
    fields = field.Fields(Interface)

    ajax_handler = 'user-profile.json'
    edit_permission = None

    dialog_class = 'modal-large'

    @property
    def title(self):
        return self.request.principal.title


@view_config(name='user-profile.json', request_type=IPyAMSLayer, renderer='json', xhr=True)
class UserProfileAJAXEditForm(AJAXEditForm, UserProfileEditForm):
    """User profile edit form, JSON renderer"""


@adapter_config(name='public_profile',
                context=(Interface, IPyAMSLayer, UserProfileEditForm),
                provides=IInnerTabForm)
class PublicProfileTabForm(InnerEditForm):
    """Public profile tab form"""

    tab_label = _("Public profile")
    legend = None

    fields = field.Fields(IPublicProfile)
    edit_permission = None

    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'

    weight = 10

    def getContent(self):
        return IPublicProfile(self.request.principal)
