#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_form.interfaces.form import IWidgetForm
from pyams_security.interfaces import IDefaultProtectionPolicy, IProtectedObject, IRole
from pyams_security.zmi.interfaces import IObjectSecurityMenu
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION, MANAGE_SECURITY_PERMISSION, MANAGE_ROLES_PERMISSION
from pyams_zmi.interfaces.menu import ISiteManagementMenu
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import DISPLAY_MODE, INPUT_MODE

# import packages
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field
from zope.interface import implementer, Interface

from pyams_security import _


#
# Object roles edit form
#

@viewlet_config(name='protected-object-roles.menu', context=IDefaultProtectionPolicy, layer=IAdminLayer,
                manager=ISiteManagementMenu, permission=VIEW_SYSTEM_PERMISSION, weight=900)
@viewletmanager_config(name='protected-object-roles.menu', layer=IAdminLayer, context=IDefaultProtectionPolicy,
                       provides=IObjectSecurityMenu)
@implementer(IObjectSecurityMenu)
class ProtectedObjectRolesMenuItem(MenuItem):
    """Protected object roles menu item"""

    label = _("Access rules")
    icon_class = 'fa-users'
    url = '#protected-object-roles.html'
    modal_target = False


@pagelet_config(name='protected-object-roles.html', context=IDefaultProtectionPolicy, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@implementer(IWidgetForm, IInnerPage)
class ProtectedObjectRolesEditForm(AdminEditForm):
    """Protected object roles edit form"""

    legend = _("Granted users roles")
    icon_css_class = 'fa fa-fw fa-users'

    @property
    def fields(self):
        return field.Fields(self.context.roles_interface)

    ajax_handler = 'protected-object-roles.json'
    edit_permission = MANAGE_ROLES_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ProtectedObjectRolesEditForm, self).updateWidgets(prefix)
        if self.mode == DISPLAY_MODE:
            return
        principals = self.request.effective_principals
        for widget in self.widgets.values():
            widget.mode = DISPLAY_MODE
            role = get_utility(IRole, name=widget.field.role_id)
            if role.managers:
                for manager in role.managers:
                    if manager in principals:
                        widget.mode = INPUT_MODE
                        continue


@view_config(name='protected-object-roles.json', context=IDefaultProtectionPolicy, request_type=IPyAMSLayer,
             permission=MANAGE_ROLES_PERMISSION, renderer='json', xhr=True)
class ProtectedObjectRolesAJAXEditForm(AJAXEditForm, ProtectedObjectRolesEditForm):
    """Protected object roles edit form, AJAX view"""


@adapter_config(context=(Interface, IPyAMSLayer, ProtectedObjectRolesEditForm), provides=IPageHeader)
class ProtectedObjectRolesEditFormHeaderAdapter(DefaultPageHeaderAdapter):
    """Protected object security edit form header adapter"""

    icon_class = 'fa fa-fw fa-users'


#
# Security policy edit form
#

@viewlet_config(name='security-policy.menu', context=IDefaultProtectionPolicy, layer=IAdminLayer,
                manager=IObjectSecurityMenu, permission=MANAGE_SECURITY_PERMISSION, weight=10)
class ProtectedObjectSecurityPolicyMenuItem(MenuItem):
    """Protected object security policy menu item"""

    label = _("Security policy...")
    icon_class = 'fa-key'
    url = 'security-policy.html'
    modal_target = True


@pagelet_config(name='security-policy.html', context=IDefaultProtectionPolicy, layer=IPyAMSLayer,
                permission=MANAGE_SECURITY_PERMISSION)
class ProtectedObjectSecurityPolicyEditForm(AdminDialogEditForm):
    """Protected object security policy edit form"""

    legend = _("Update security policy")
    icon_css_class = 'fa fa-fw fa-key'

    fields = field.Fields(IProtectedObject)
    ajax_handler = 'security-policy.json'
    edit_permission = MANAGE_SECURITY_PERMISSION

    dialog_class = 'modal-large'


@view_config(name='security-policy.json', context=IDefaultProtectionPolicy, request_type=IPyAMSLayer,
             permission=MANAGE_SECURITY_PERMISSION, renderer='json', xhr=True)
class ProtectedObjectSecurityPolicyAJAXEditForm(AJAXEditForm, ProtectedObjectSecurityPolicyEditForm):
    """Protected object security policy edit form, JSON renderer"""
